// Hispaniola -- JSB
// March 2006
// Nov 06 - YP update
// Main entry point for random map script

include "mercenaries.xs";
include "ypAsianInclude.xs";
include "ypKOTHInclude.xs";

void main(void)
{
	// --------------- Make load bar move. ----------------------------------------------------------------------------
	rmSetStatusText("",0.10);


	// Set size of map
	int playerTiles=17500;
	if (cNumberNonGaiaPlayers >2)
		playerTiles = 13500;
	if (cNumberNonGaiaPlayers >4)
		playerTiles = 11000;
	if (cNumberNonGaiaPlayers >6)
		playerTiles = 10000;
	int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
	rmSetMapSize(size, size);

	// Set up default water type.
	rmSetSeaLevel(-10.0);
	rmSetSeaType("Ceylon coast");
	rmSetOceanReveal(true);
	rmSetWindMagnitude(20.0);
	rmSetMapType("Yukon");
	rmSetMapType("grass");
	rmSetMapType("water");
	rmSetLightingSet("Great Lakes Winter");

	rmTerrainInitialize("water");

	int numTries = -1;
	int classPlayer=rmDefineClass("player");
	int classIsland=rmDefineClass("island");
	rmDefineClass("classForest");
	rmDefineClass("importantItem");
	rmDefineClass("natives");
	rmDefineClass("classSocket");
	rmDefineClass("canyon");
	rmDefineClass("classCliff");
	rmDefineClass("startingUnit");

   // -------------Define constraints----------------------------------------


	// Player area constraint.
	int avoidTC=rmCreateTypeDistanceConstraint("stay away from TC", "TownCenter", 30.0);
	int avoidTCFar=rmCreateTypeDistanceConstraint("stay away from TC Far", "TownCenter", 60.0);
	int avoidTCFar1=rmCreateTypeDistanceConstraint("stay away from TC Far1", "TownCenter", 40.0);
	int avoidStartingUnits=rmCreateClassDistanceConstraint("objects avoid starting units", rmClassID("startingUnit"), 8.0);

	// Ressources Constraint
	int avoidCoin = rmCreateTypeDistanceConstraint("avoid coin", "MineTin", 50.0);
	int avoidCoinStart = rmCreateTypeDistanceConstraint("avoid coin 2", "MineTin", 20.0);
	int avoidRandomTurkeys=rmCreateTypeDistanceConstraint("avoid random turkeys", "turkey", 50.0);
	int forestRightConstraint=rmCreateClassDistanceConstraint("forest vs. forest right", rmClassID("classForest"), 5.0);
	int forestID = -1;
	int forestConstraint=rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 35.0);
	int avoidMuskOx = rmCreateTypeDistanceConstraint("avoid muskox", "muskOx", 48.0);
	int avoidMoose = rmCreateTypeDistanceConstraint("avoid moose", "moose", 18.0);
	int avoidCaribou = rmCreateTypeDistanceConstraint("avoid caribou", "caribou", 48.0);
	int avoidCaribouShort = rmCreateTypeDistanceConstraint("avoid caribou short", "caribou", 15.0);
	int avoidCoinShort=rmCreateTypeDistanceConstraint("avoids coin short", "gold", 8.0);

	// Generic Contraints
	int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 5.0);
	int avoidImpassableLandShort = rmCreateTerrainDistanceConstraint("short avoid impassable land like Arkansas for tree", "Land", false, 3.0);
	int avoidTradeRoute = rmCreateTradeRouteDistanceConstraint("trade route", 6.0);
	int avoidSocket = rmCreateClassDistanceConstraint("avoid socket", rmClassID("classSocket"), 8.0);
	int avoidImportantItem = rmCreateClassDistanceConstraint("secrets etc avoid each other", rmClassID("importantItem"), 50.0);
	int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 4.0);
	int avoidCliffsShort=rmCreateClassDistanceConstraint("avoid Cliffs short", rmClassID("classCliff"), 4.0);
	int avoidCliffs=rmCreateClassDistanceConstraint("avoid Cliffs", rmClassID("classCliff"), 10.0);
	int avoidNugget=rmCreateTypeDistanceConstraint("nugget avoid nugget", "AbstractNugget", 35.0);
	int avoidNuggetStart=rmCreateTypeDistanceConstraint("nugget avoid nugget start", "AbstractNugget", 20.0);

	// Custom Contraints
	int Promontoir=rmCreatePieConstraint("promontoir", 0.5, 0.77, rmXFractionToMeters(0.04), rmXFractionToMeters(0.8), rmDegreesToRadians(0), rmDegreesToRadians(360));
	int PromontoirHunts=rmCreatePieConstraint("eviter le petit promontoir", 0.5, 0.75, rmXFractionToMeters(0.06), rmXFractionToMeters(0.8), rmDegreesToRadians(0), rmDegreesToRadians(360));
	int ContrePromontoir=rmCreatePieConstraint("contre promontoir", 0.5, 0.91, rmXFractionToMeters(0.08), rmXFractionToMeters(0.9), rmDegreesToRadians(0), rmDegreesToRadians(360));
	int avoidPic = rmCreatePieConstraint("avoid Pic", 0.5, 0.15, rmXFractionToMeters(0.08), rmXFractionToMeters(0.5), rmDegreesToRadians(0), rmDegreesToRadians(360));

	chooseMercs();

// ----------------------------------------- Building the layout ------------------------------------------------------
	float coeff = 1.6;

	int lePuit=rmCreateArea("le trou en haut");
	rmSetAreaLocation(lePuit, 0.38, 0.75);
	rmAddAreaInfluenceSegment(lePuit, 0.3, 0.75, 0.7, 0.75);
	rmSetAreaSize(lePuit, 0.02*coeff, 0.02*coeff);
	rmSetAreaCliffType(lePuit, "SPC Mountain Pass");
    rmSetAreaCliffEdge(lePuit, 1, 1.0);
    rmSetAreaCliffPainting(lePuit, true, true, true, 1.5, true);
    rmSetAreaCliffHeight(lePuit, -20, 0.0, 0.5);
    rmSetAreaHeightBlend(lePuit, 1);
    rmSetAreaSmoothDistance(lePuit, 10);
	rmAddAreaConstraint(lePuit, Promontoir);
    rmSetAreaCoherence(lePuit, 0.75);
	rmBuildArea(lePuit);

	int avanceeBasGauche=rmCreateArea("avancee en bas trou gauche");
	rmSetAreaLocation(avanceeBasGauche, 0.35, 0.11);
	rmAddAreaInfluenceSegment(avanceeBasGauche, 0.2, 0.11, 0.43, 0.11);
	rmSetAreaSize(avanceeBasGauche, 0.017*coeff, 0.017*coeff);
	rmSetAreaCliffType(avanceeBasGauche, "SPC Mountain Pass");
    rmSetAreaCliffEdge(avanceeBasGauche, 1, 1.0);
    rmSetAreaCliffPainting(avanceeBasGauche, true, true, true, 1.5, true);
    rmSetAreaCliffHeight(avanceeBasGauche, -25, 0.0, 0.5);
    rmSetAreaHeightBlend(avanceeBasGauche, 1);
    rmAddAreaToClass(avanceeBasGauche, rmClassID("classCliff"));
    rmSetAreaSmoothDistance(avanceeBasGauche, 10);
    rmSetAreaCoherence(avanceeBasGauche, 0.8);
	rmBuildArea(avanceeBasGauche);

	int avanceeBasDroite=rmCreateArea("avancee en bas trou droit");
	rmSetAreaLocation(avanceeBasDroite, 0.65, 0.12);
	rmAddAreaInfluenceSegment(avanceeBasDroite, 0.57, 0.1, 0.8, 0.11);
	rmSetAreaSize(avanceeBasDroite, 0.017*coeff, 0.017*coeff);
	rmSetAreaCliffType(avanceeBasDroite, "SPC Mountain Pass");
    rmSetAreaCliffEdge(avanceeBasDroite, 1, 1.0);
    rmSetAreaCliffPainting(avanceeBasDroite, true, true, true, 1.5, true);
    rmSetAreaCliffHeight(avanceeBasDroite, -25, 0.0, 0.5);
    rmSetAreaHeightBlend(avanceeBasDroite, 1);
    rmAddAreaToClass(avanceeBasDroite, rmClassID("classCliff"));
    rmSetAreaSmoothDistance(avanceeBasDroite, 10);
    rmSetAreaCoherence(avanceeBasDroite, 0.8);
	rmBuildArea(avanceeBasDroite);


// ************* TradeRoute & Center Island **************************************
   int tradeRouteID = rmCreateTradeRoute();
   int socketID=rmCreateObjectDef("sockets to dock Trade Posts");
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID);
   rmAddObjectDefItem(socketID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socketID, true);
   rmAddObjectDefToClass(socketID, rmClassID("classSocket"));
   rmSetObjectDefMinDistance(socketID, 0.0);
   rmSetObjectDefMaxDistance(socketID, 8.0);

   rmAddTradeRouteWaypoint(tradeRouteID, 0.15, 0.4);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.3, 0.34, 4, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.5, 0.25, 4, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.7, 0.34, 4, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.85, 0.4, 4, 1);

   bool placedTradeRoute = rmBuildTradeRoute(tradeRouteID, "snow");

	// Build The ISLAND
	int centerIsland=rmCreateArea("The Island");
	rmSetAreaLocation(centerIsland, 0.5, 0.5);
	rmSetAreaSize(centerIsland, 0.33*coeff, 0.33*coeff);
    rmSetAreaWarnFailure(centerIsland, false);
	rmSetAreaCliffType(centerIsland, "SPC Mountain Pass");
    rmSetAreaCliffEdge(centerIsland, 1, 1, 0, 0, 1);
	rmSetAreaMinBlobs(centerIsland, 2);
	rmAddAreaInfluenceSegment(centerIsland, 0.5, 0.47, 0.5, 0.53);
	rmSetAreaTerrainType(centerIsland, "yukon\ground1_yuk");
	rmSetAreaMix(centerIsland, "snow");
    rmSetAreaCliffPainting(centerIsland, true, true, true, 10, true);
    rmSetAreaCliffHeight(centerIsland, 18, 0.0, 0.5);
    rmSetAreaHeightBlend(centerIsland, 1);
    rmSetAreaSmoothDistance(centerIsland, 10);
    rmSetAreaCoherence(centerIsland, 0.88);
	rmAddAreaConstraint(centerIsland, ContrePromontoir);
	rmBuildArea(centerIsland);

	int stayCenterIsland=rmCreateAreaConstraint("rester sur l'ile (ne pas aller dans l'eau)", centerIsland);

	int icePatch = rmCreateArea("ice on the top ramp");
   rmSetAreaLocation(icePatch, 0.5, 0.82);
   rmAddAreaInfluenceSegment(icePatch, 0.7, 0.82, 0.3, 0.82);
   rmSetAreaWarnFailure(icePatch, false);
   rmSetAreaSize(icePatch, 0.06*coeff, 0.06*coeff);
   rmSetAreaCoherence(icePatch, 0.85);
   rmSetAreaSmoothDistance(icePatch, 1);
   rmAddAreaToClass(icePatch, rmClassID("classPatch"));
   rmSetAreaTerrainType(icePatch, "great_lakes\ground_ice1_gl");
   rmBuildArea(icePatch);

	int avoidIcePatch = rmCreateAreaDistanceConstraint("avoid icePatch", icePatch, 5);

	int NewPatch = rmCreateArea("new Patch for the map");
    rmSetAreaLocation(NewPatch, 0.5, 0.5);
    rmSetAreaWarnFailure(NewPatch, false);
    rmSetAreaSize(NewPatch, 0.5*coeff, 0.5*coeff);
    rmSetAreaCoherence(NewPatch, 0.85);
    rmSetAreaSmoothDistance(NewPatch, 1);
    rmAddAreaToClass(NewPatch, rmClassID("classPatch"));
	rmAddAreaConstraint(NewPatch, stayCenterIsland);
	rmAddAreaConstraint(NewPatch, avoidIcePatch);
    rmSetAreaMix(NewPatch, "greatlakes_snow");
    rmBuildArea(NewPatch);

	int patchInIce = rmCreateArea("patch in ice");
	rmSetAreaLocation(patchInIce, 0.5, 0.79);
	rmSetAreaWarnFailure(patchInIce, false);
	rmSetAreaSize(patchInIce, 0.002, 0.002);
	rmSetAreaCoherence(patchInIce, 0.85);
	rmSetAreaSmoothDistance(patchInIce, 1);
	rmAddAreaToClass(patchInIce, rmClassID("classPatch"));
	rmSetAreaMix(patchInIce, "greatlakes_snow");
	rmBuildArea(patchInIce);

	vector socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.1);
    rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
	socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.5);
    rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
    socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.9);
    rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);


//****************** La pente *************************************
	int rightPart=rmCreateArea("la petite partie penchee en bas");
	rmSetAreaSize(rightPart, 0.23, 0.23);
	rmSetAreaLocation(rightPart, 0.5, 0.1);
	rmSetAreaCliffType(rightPart, "SPC Mountain Pass");
	rmAddAreaInfluenceSegment(rightPart, 0.1, 0.1, 0.9, 0.1);
    rmSetAreaCliffEdge(rightPart, 1, 0.0);
	rmSetAreaTerrainType(rightPart, "yukon\ground1_yuk");
    rmSetAreaCliffPainting(rightPart, true, true, true, 10, true);
    rmSetAreaCliffHeight(rightPart, -6, 0.0, 1);
 //   rmSetAreaHeightBlend(rightPart, 0);
    rmSetAreaSmoothDistance(rightPart, 5);
    rmSetAreaCoherence(rightPart, 0.9);
	rmBuildArea(rightPart);

	int stayRightPart=rmCreateAreaConstraint("rester sur la partie penchee", rightPart);

	for (j=0; < rmRandInt(cNumberNonGaiaPlayers*20,cNumberNonGaiaPlayers*22)) //18,20
	{
		int seTree = rmCreateObjectDef("south west tree"+j);
		rmAddObjectDefItem(seTree, "TreeYukon", rmRandInt(1,2), .0); // 1,2
		rmSetObjectDefMinDistance(seTree, 0);
		rmSetObjectDefMaxDistance(seTree, rmXFractionToMeters(0.5));
		//rmAddObjectDefToClass(seTree, rmClassID("classForest"));
		rmAddObjectDefConstraint(seTree, stayRightPart);
		rmAddObjectDefConstraint(seTree, avoidImpassableLandShort);
		rmAddObjectDefConstraint(seTree, avoidSocket);
		rmAddObjectDefConstraint(seTree, avoidPic);
		rmAddObjectDefConstraint(seTree, forestRightConstraint);
		rmPlaceObjectDefAtLoc(seTree, 0, 0.50, 0.50);
	}


// *********** 2 Cliffs area *****************
/*	int LeftClliff1=rmCreateArea("Left Cliff1");
	rmSetAreaLocation(LeftClliff1, 0.64, 0.65);
	rmSetAreaSize(LeftClliff1, 0.0025, 0.0025);
    rmSetAreaWarnFailure(LeftClliff1, false);
	rmSetAreaCliffType(LeftClliff1, "SPC Mountain Pass");
    rmSetAreaCliffEdge(LeftClliff1, 1, 1);
	rmAddAreaInfluenceSegment(LeftClliff1, 0.64, 0.65, 0.64, 0.7);
    rmSetAreaCliffPainting(LeftClliff1, false, true, true, 1.5, true);
    rmSetAreaCliffHeight(LeftClliff1, 8, 0.0, 0.5);
    rmSetAreaHeightBlend(LeftClliff1, 1);
    rmAddAreaToClass(LeftClliff1, rmClassID("classCliff"));
    rmSetAreaSmoothDistance(LeftClliff1, 5);
    rmSetAreaCoherence(LeftClliff1, 0.5);
	rmAddAreaConstraint(LeftClliff1,stayCenterIsland);
	rmBuildArea(LeftClliff1);*/


	int LeftCliff1A=rmCreateArea("Left Cliff1A");
	rmSetAreaLocation(LeftCliff1A, 0.64, 0.52);
	rmSetAreaSize(LeftCliff1A, 0.0025, 0.0025);
    rmSetAreaWarnFailure(LeftCliff1A, false);
	rmSetAreaCliffType(LeftCliff1A, "SPC Mountain Pass");
    rmSetAreaCliffEdge(LeftCliff1A, 1, 1);
	rmAddAreaInfluenceSegment(LeftCliff1A, 0.64, 0.50, 0.64, 0.55);
    rmSetAreaCliffPainting(LeftCliff1A, false, true, true, 1.5, true);
    rmSetAreaCliffHeight(LeftCliff1A, 8, 0.0, 0.5);
    rmSetAreaHeightBlend(LeftCliff1A, 1);
    rmAddAreaToClass(LeftCliff1A, rmClassID("classCliff"));
    rmSetAreaSmoothDistance(LeftCliff1A, 5);
    rmSetAreaCoherence(LeftCliff1A, 0.5);
	rmAddAreaConstraint(LeftCliff1A,stayCenterIsland);
	rmBuildArea(LeftCliff1A);
/*
	int LeftCliff2=rmCreateArea("Left Cliff2");
	rmSetAreaLocation(LeftCliff2, 0.36, 0.65);
	rmSetAreaSize(LeftCliff2, 0.0025, 0.0025);
    rmSetAreaWarnFailure(LeftCliff2, false);
	rmSetAreaCliffType(LeftCliff2, "SPC Mountain Pass");
    rmSetAreaCliffEdge(LeftCliff2, 1, 1);
	rmAddAreaInfluenceSegment(LeftCliff2, 0.36, 0.65, 0.36, 0.7);
    rmSetAreaCliffPainting(LeftCliff2, false, true, true, 1.5, true);
    rmSetAreaCliffHeight(LeftCliff2, 8, 0.0, 0.5);
    rmSetAreaHeightBlend(LeftCliff2, 1);
    rmAddAreaToClass(LeftCliff2, rmClassID("classCliff"));
    rmSetAreaSmoothDistance(LeftCliff2, 5);
    rmSetAreaCoherence(LeftCliff2, 0.5);
	rmAddAreaConstraint(LeftCliff2,stayCenterIsland);
	rmBuildArea(LeftCliff2);*/


	int LeftCliff2A=rmCreateArea("Left Cliff2A");
	rmSetAreaLocation(LeftCliff2A, 0.36, 0.52);
	rmSetAreaSize(LeftCliff2A, 0.0025, 0.0025);
    rmSetAreaWarnFailure(LeftCliff2A, false);
	rmSetAreaCliffType(LeftCliff2A, "SPC Mountain Pass");
    rmSetAreaCliffEdge(LeftCliff2A, 1, 1);
	rmAddAreaInfluenceSegment(LeftCliff2A, 0.36, 0.5, 0.36, 0.55);
    rmSetAreaCliffPainting(LeftCliff2A, false, true, true, 1.5, true);
    rmSetAreaCliffHeight(LeftCliff2A, 8, 0.0, 0.5);
    rmSetAreaHeightBlend(LeftCliff2A, 1);
    rmAddAreaToClass(LeftCliff2A, rmClassID("classCliff"));
    rmSetAreaSmoothDistance(LeftCliff2A, 5);
    rmSetAreaCoherence(LeftCliff2A, 0.5);
	rmAddAreaConstraint(LeftCliff2A,stayCenterIsland);
	rmBuildArea(LeftCliff2A);

 	   int betweenCliffPatch = rmCreateArea("patch between the two cliffs");
   rmSetAreaLocation(betweenCliffPatch, 0.5, 0.69);
   rmAddAreaInfluenceSegment(betweenCliffPatch, 0.36, 0.69, 0.64, 0.69);
   rmSetAreaWarnFailure(betweenCliffPatch, false);
   rmSetAreaSize(betweenCliffPatch, 0.008*coeff, 0.008*coeff);
   rmSetAreaCoherence(betweenCliffPatch, 0.85);
   rmSetAreaSmoothDistance(betweenCliffPatch, 1);
   rmAddAreaToClass(betweenCliffPatch, rmClassID("classPatch"));
   rmSetAreaTerrainType(betweenCliffPatch, "yukon\ground3x_yuk");
   rmAddAreaConstraint(betweenCliffPatch, avoidCliffsShort);
  // rmSetAreaMix(betweenCliffPatch, "greatlakes_snow");
   rmBuildArea(betweenCliffPatch);


     int inuksukObject = rmCreateObjectDef("inuksuk Object");
	rmAddObjectDefItem(inuksukObject, "inuksuk", 1, 1.0);
	rmSetObjectDefMinDistance(inuksukObject, 0);
	rmSetObjectDefMaxDistance(inuksukObject, 0);
	rmSetObjectDefForceFullRotation(inuksukObject, true);
	rmPlaceObjectDefAtLoc(inuksukObject, 0, 0.5, 0.61);


// ********* The little Way ***************************
   	int middleForest = rmCreateArea("middle forest");
	rmSetAreaLocation(middleForest, 0.5, 0.4);
	rmAddAreaInfluenceSegment(middleForest, 0.55, 0.4, 0.45, 0.4);
	rmSetAreaWarnFailure(middleForest, false);
	rmSetAreaSize(middleForest, 0.011*coeff, 0.011*coeff);
	rmSetAreaBaseHeight(middleForest, 8.0);
	rmSetAreaMinBlobDistance(middleForest, 14.0);
	rmSetAreaMaxBlobDistance(middleForest, 30.0);
	rmSetAreaCoherence(middleForest, 0.9);
	rmSetAreaSmoothDistance(middleForest, 5);
	rmSetAreaHeightBlend(middleForest, 2);
	//rmAddAreaToClass(middleForest, rmClassID("classForest"));
	rmBuildArea(middleForest);

	int littleGap = rmCreateArea("middle gap");
	rmSetAreaLocation(littleGap, 0.5, 0.4);
	rmAddAreaInfluenceSegment(littleGap, 0.62, 0.4, 0.38, 0.4);
	rmSetAreaWarnFailure(littleGap, false);
	rmSetAreaSize(littleGap, 0.006*coeff, 0.006*coeff);
	rmSetAreaBaseHeight(littleGap, 5.0);
	rmSetAreaTerrainType(littleGap, "yukon\ground1_yuk");
	rmSetAreaMinBlobDistance(littleGap, 14.0);
	rmSetAreaMaxBlobDistance(littleGap, 30.0);
	rmSetAreaCoherence(littleGap, 0.9);
	rmSetAreaSmoothDistance(littleGap, 5);
	rmSetAreaHeightBlend(littleGap, 0);
	rmBuildArea(littleGap);


    int stayInMiddleForest =  rmCreateAreaConstraint("stay in middle forest", middleForest);
	int avoidMiddleRoad = rmCreateAreaDistanceConstraint("avoid the little way", littleGap, 5);

   	for (j=0; < rmRandInt(cNumberNonGaiaPlayers*25,cNumberNonGaiaPlayers*26)) //X marks the spot
	{
		int middleTree = rmCreateObjectDef("middle tree"+j);
		rmAddObjectDefItem(middleTree, "TreePatagoniaSnow", rmRandInt(1,2), 4.0); // 1,2
		rmSetObjectDefMinDistance(middleTree, 0);
		rmSetObjectDefMaxDistance(middleTree, rmXFractionToMeters(0.5));
		//rmAddObjectDefToClass(middleTree, rmClassID("classForest"));
		rmAddObjectDefConstraint(middleTree, stayInMiddleForest);
		rmAddObjectDefConstraint(middleTree, avoidMiddleRoad);
		rmAddObjectDefConstraint(middleTree, avoidImpassableLandShort);
		rmPlaceObjectDefAtLoc(middleTree, 0, 0.50, 0.50);
	}

   rmSetStatusText("",0.40);

// --------------------------------------------- Starting Ressources --------------------------



if (cNumberNonGaiaPlayers==2)
{
	if (rmRandFloat(0,1)<0.5)
	{
   rmSetPlacementTeam(0);
   rmSetPlacementSection(0.2, 0.28);
   rmPlacePlayersCircular(0.3, 0.3, 0);

   rmSetPlacementTeam(1);
   rmSetPlacementSection(0.8, 0.78);
   rmPlacePlayersCircular(0.3, 0.3, 0);
	}
	else
	{
   rmSetPlacementTeam(1);
   rmSetPlacementSection(0.2, 0.28);
   rmPlacePlayersCircular(0.3, 0.3, 0);

   rmSetPlacementTeam(0);
   rmSetPlacementSection(0.8, 0.78);
   rmPlacePlayersCircular(0.3, 0.3, 0);
	}
}
else
{
   rmSetPlacementTeam(0);
   rmSetPlacementSection(0.16, 0.27);
   rmPlacePlayersCircular(0.3, 0.3, 0);

   rmSetPlacementTeam(1);
   rmSetPlacementSection(0.73, 0.84);
   rmPlacePlayersCircular(0.3, 0.3, 0);

}


   int avoidEdgeIsland=rmCreateCliffEdgeDistanceConstraint("avoid center island edge", centerIsland, 3);
   for(i=1; <cNumberPlayers)
   {
		// Create the area.
		int id=rmCreateArea("Player"+i);
		// Assign to the player.
		rmSetPlayerArea(i, id);
			// Set the size.
		rmSetAreaSize(id, 0.04, 0.04);
		rmAddAreaToClass(id, classPlayer);
		rmSetAreaMinBlobs(id, 1);
		rmSetAreaMaxBlobs(id, 1);
		rmSetAreaLocPlayer(id, i);
		rmSetAreaCoherence(id, 0.8);
		rmSetAreaTerrainType(id, "yukon\ground3x_yuk");
		rmAddAreaConstraint(id, avoidCliffsShort);
		rmAddAreaConstraint(id, avoidEdgeIsland);
		rmSetAreaWarnFailure(id, false);
		rmBuildArea(id);
   }


   for(i=1; <cNumberPlayers)
   {
		// Create the area.
		int id1=rmCreateArea("For players"+i);
		// Assign to the player.
		rmSetPlayerArea(i, id1);
			// Set the size.
		rmSetAreaSize(id1, 0.015, 0.015);
		rmAddAreaToClass(id1, classPlayer);
		rmSetAreaMinBlobs(id1, 10);
		rmSetAreaMaxBlobs(id1, 12);
		rmSetAreaLocPlayer(id1, i);
		rmSetAreaCoherence(id1, 0.3);
		rmSetAreaTerrainType(id1, "yukon\ground3_yuk");
		rmAddAreaConstraint(id1, avoidCliffsShort);
		rmAddAreaConstraint(id1, avoidEdgeIsland);
		rmSetAreaSmoothDistance(id1,1);
		rmSetAreaWarnFailure(id1, false);
		rmBuildArea(id1);
   }

   	int startingUnits = rmCreateStartingUnitsObjectDef(5.0);

	int startingTCID = rmCreateObjectDef("startingTC");
	if ( rmGetNomadStart())
	{
		rmAddObjectDefItem(startingTCID, "CoveredWagon", 1, 0.0);
	}
	else
	{
		rmAddObjectDefItem(startingTCID, "TownCenter", 1, 0.0);
	}
	rmAddObjectDefToClass(startingTCID, rmClassID("startingUnit"));
	rmSetObjectDefMinDistance(startingTCID, 0.0);
	rmSetObjectDefMaxDistance(startingTCID, 0.0);

	int StartCaribou=rmCreateObjectDef("starting caribou");
	rmAddObjectDefItem(StartCaribou, "caribou", rmRandInt(7,8), 6.0);
	rmSetObjectDefMinDistance(StartCaribou, 8.0);
	rmSetObjectDefMaxDistance(StartCaribou, 10.0);
	rmSetObjectDefCreateHerd(StartCaribou, true);
	rmAddObjectDefConstraint(StartCaribou, avoidImpassableLand);
	rmAddObjectDefConstraint(StartCaribou, avoidStartingUnits);
	rmAddObjectDefConstraint(StartCaribou, avoidSocket);

	int StartMoose=rmCreateObjectDef("starting moose");
	rmAddObjectDefItem(StartMoose, "moose", rmRandInt(7,8), 9.0);
	rmAddObjectDefToClass(StartMoose, rmClassID("startingUnit"));
	rmSetObjectDefMinDistance(StartMoose, 27.0);
	rmSetObjectDefMaxDistance(StartMoose, 29.0);
	rmSetObjectDefCreateHerd(StartMoose, true);
	rmAddObjectDefConstraint(StartMoose, avoidImpassableLand);
	rmAddObjectDefConstraint(StartMoose, avoidStartingUnits);
	rmAddObjectDefConstraint(StartMoose, avoidCaribouShort);
	rmAddObjectDefConstraint(StartMoose, avoidSocket);

	int startSilverID = rmCreateObjectDef("first mine");
	rmAddObjectDefItem(startSilverID, "MineTin", 1, 5.0);
	rmAddObjectDefToClass(startSilverID, rmClassID("startingUnit"));
	rmSetObjectDefMinDistance(startSilverID, 13.0);
	rmSetObjectDefMaxDistance(startSilverID, 15.0);

		int startSilverID2 = rmCreateObjectDef("first mine2");
	rmAddObjectDefItem(startSilverID2, "MineTin", 1, 5.0);
	rmAddObjectDefToClass(startSilverID2, rmClassID("startingUnit"));
	rmAddObjectDefConstraint(startSilverID2, avoidCoinStart);
	rmSetObjectDefMinDistance(startSilverID2, 22.0);
	rmSetObjectDefMaxDistance(startSilverID2, 23.0);


	int StartAreaTreeID=rmCreateObjectDef("starting trees");
	rmAddObjectDefItem(StartAreaTreeID, "TreeYukon", 2, 4.0);
	rmSetObjectDefMinDistance(StartAreaTreeID, 11);
	rmSetObjectDefMaxDistance(StartAreaTreeID, 18);
	rmAddObjectDefToClass(StartAreaTreeID, rmClassID("startingUnit"));
	rmAddObjectDefConstraint(StartAreaTreeID, avoidImpassableLand);
	rmAddObjectDefConstraint(StartAreaTreeID, avoidStartingUnits);
	rmAddObjectDefConstraint(StartAreaTreeID, avoidSocket);

	int playerNuggetID=rmCreateObjectDef("player nugget");
	rmAddObjectDefItem(playerNuggetID, "nugget", 1, 0.0);
	rmAddObjectDefToClass(playerNuggetID, rmClassID("startingUnit"));
    rmSetObjectDefMinDistance(playerNuggetID, 23.0);
    rmSetObjectDefMaxDistance(playerNuggetID, 28.0);
	rmAddObjectDefConstraint(playerNuggetID, avoidNuggetStart);
	rmAddObjectDefConstraint(playerNuggetID, avoidTradeRoute);
	rmAddObjectDefConstraint(playerNuggetID, avoidSocket);
	rmAddObjectDefConstraint(playerNuggetID, avoidStartingUnits);
	rmAddObjectDefConstraint(playerNuggetID, avoidAll);

	 	for(i=1; <=cNumberNonGaiaPlayers)
	{
		rmPlaceObjectDefAtLoc(startingTCID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

		if(ypIsAsian(i) && rmGetNomadStart() == false)
      		rmPlaceObjectDefAtLoc(ypMonasteryBuilder(i, 1), i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmPlaceObjectDefAtLoc(startSilverID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmPlaceObjectDefAtLoc(startSilverID2, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmPlaceObjectDefAtLoc(StartCaribou, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmPlaceObjectDefAtLoc(StartMoose, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

		rmPlaceObjectDefAtLoc(StartAreaTreeID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmPlaceObjectDefAtLoc(StartAreaTreeID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmPlaceObjectDefAtLoc(StartAreaTreeID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmSetNuggetDifficulty(1,1);
		rmPlaceObjectDefAtLoc(playerNuggetID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
		rmPlaceObjectDefAtLoc(playerNuggetID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	}

	 rmSetStatusText("",0.60);
// ----------------------------------------------------- Ressources ------------------------------------------------------------------
// ************************ Mines *********************************

	int silverID = rmCreateObjectDef("mines with high variation");
	rmAddObjectDefItem(silverID, "MineTin", 1, 0);
	rmSetObjectDefMinDistance(silverID, 0.0);
	rmSetObjectDefMaxDistance(silverID, 12);
	rmAddObjectDefConstraint(silverID, avoidCliffsShort);
	rmAddObjectDefConstraint(silverID, avoidImpassableLandShort);
	rmAddObjectDefConstraint(silverID, avoidSocket);
	rmAddObjectDefConstraint(silverID, avoidTradeRoute);


   float mineVariation = rmRandFloat(0,1);
   float mineBonus = rmRandFloat(0,1);

if (cNumberNonGaiaPlayers==2)
{
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.5, 0.11);	// sur la jetee au sud

	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.5, 0.71); // entre les deux falaises

	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.7, 0.25); // sur la partie basse
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.3, 0.25);
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.5, 0.5); // entre les deux falaises

	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.37, 0.8); // sur la petite partie gelee
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.63, 0.8);

		if (mineBonus<0)
		{
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.77, 0.77);
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.23, 0.77);
		}
		else if (mineBonus<0)
		{
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.67, 0.47);
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.33, 0.47);
		}
		else
		{
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.81, 0.42);
	rmSetObjectDefForceFullRotation(silverID, true);
	rmPlaceObjectDefAtLoc(silverID, 0, 0.19, 0.42);
		}
}
else
{
	   for (i=0; <cNumberNonGaiaPlayers*4)
      {
   	int silverIDT = rmCreateObjectDef("mines for 1V1"+i);
	rmAddObjectDefItem(silverIDT, "MineTin", 1, 0);
	rmSetObjectDefMinDistance(silverIDT, 0.0);
	rmSetObjectDefMaxDistance(silverIDT, rmZFractionToMeters(0.45));
	rmAddObjectDefConstraint(silverIDT, avoidTradeRoute);
	rmAddObjectDefConstraint(silverIDT, avoidCliffsShort);
	rmAddObjectDefConstraint(silverIDT, avoidTCFar1);
	rmAddObjectDefConstraint(silverIDT, avoidSocket);
    rmAddObjectDefConstraint(silverIDT, avoidImpassableLandShort);
	rmAddObjectDefConstraint(silverIDT, avoidCoin);
	rmAddObjectDefConstraint(silverIDT, stayCenterIsland);
	rmSetObjectDefForceFullRotation(silverIDT, true);
	rmPlaceObjectDefAtLoc(silverIDT, 0, 0.5, 0.5);
      }
}

// ******************** Forests *****************************

int avoidMiddleForest = rmCreateAreaDistanceConstraint("avoid the center forest", middleForest, 25);
int avoidRightPart = rmCreateAreaDistanceConstraint("avoid the right Part", rightPart, 1);
int avoidRightPart1 = rmCreateAreaDistanceConstraint("avoid the right Part1", avanceeBasGauche, 0);

	for (i=0; < cNumberNonGaiaPlayers*8)
	{
		forestID = rmCreateArea("forest"+i);
		rmSetAreaWarnFailure(forestID, false);
		rmSetAreaSize(forestID, rmAreaTilesToFraction(50), rmAreaTilesToFraction(70));
		rmSetAreaMinBlobs(forestID, 1);
		rmSetAreaMaxBlobs(forestID, 1);
		rmSetAreaMinBlobDistance(forestID, 14.0);
		rmSetAreaMaxBlobDistance(forestID, 30.0);
		rmSetAreaCoherence(forestID, 0.1);
		rmSetAreaSmoothDistance(forestID, 5);
		rmAddAreaToClass(forestID, rmClassID("classForest"));
		rmAddAreaConstraint(forestID, avoidTC);
		rmAddAreaConstraint(forestID, forestConstraint);
		rmAddAreaConstraint(forestID, stayCenterIsland);
		rmAddAreaConstraint(forestID, avoidIcePatch);
		rmAddAreaConstraint(forestID, avoidTradeRoute);
		rmAddAreaConstraint(forestID, avoidMiddleForest);
		rmAddAreaConstraint(forestID, avoidRightPart);
		rmAddAreaConstraint(forestID, avoidImpassableLandShort);
		rmAddAreaConstraint(forestID, avoidCoinShort);
		rmBuildArea(forestID);

		int stayInNorthForest = rmCreateAreaMaxDistanceConstraint("stay in north forest"+i, forestID, 0);

		for (j=0; < rmRandInt(6,7))
		{
			int northtreeID = rmCreateObjectDef("forest tree"+i+j);
			rmAddObjectDefItem(northtreeID, "TreePatagoniaSnow", rmRandInt(1,2), 1.0); // cluster distance 9.0
			rmSetObjectDefMinDistance(northtreeID, 0);
			rmSetObjectDefMaxDistance(northtreeID, rmXFractionToMeters(0.5));
			rmAddObjectDefToClass(northtreeID, rmClassID("classForest"));
			rmAddObjectDefConstraint(northtreeID, stayInNorthForest);
			rmAddObjectDefConstraint(northtreeID, avoidImpassableLandShort);
			rmPlaceObjectDefAtLoc(northtreeID, 0, 0.40, 0.40);
		}
	}
// **************************** Hunts*******************************
	int CaribouID=rmCreateObjectDef("cariou herd");
	rmAddObjectDefItem(CaribouID, "caribou", rmRandInt(9,10), 10.0);
	rmSetObjectDefMinDistance(CaribouID, 0.0);
	rmSetObjectDefMaxDistance(CaribouID, rmXFractionToMeters(0.5));
    rmAddObjectDefConstraint(CaribouID, avoidAll);
    rmAddObjectDefConstraint(CaribouID, avoidCoinShort);
	rmAddObjectDefConstraint(CaribouID, avoidCaribou);
	rmAddObjectDefConstraint(CaribouID, avoidMuskOx);
	rmAddObjectDefConstraint(CaribouID, avoidMoose);
	rmAddObjectDefConstraint(CaribouID, avoidSocket);
	rmAddObjectDefConstraint(CaribouID, avoidTCFar);
	rmAddObjectDefConstraint(CaribouID, avoidCliffs);
//	rmAddObjectDefConstraint(CaribouID, PromontoirHunts);
	rmAddObjectDefConstraint(CaribouID, stayCenterIsland);
	rmSetObjectDefCreateHerd(CaribouID, true);
	rmPlaceObjectDefAtLoc(CaribouID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*3);

	int MuskOXID=rmCreateObjectDef("musk Ox herd");
	rmAddObjectDefItem(MuskOXID, "muskOx", rmRandInt(9,10), 10.0);
	rmSetObjectDefMinDistance(MuskOXID, 0.0);
	rmSetObjectDefMaxDistance(MuskOXID, rmXFractionToMeters(0.5));
    rmAddObjectDefConstraint(MuskOXID, avoidAll);
    rmAddObjectDefConstraint(MuskOXID, avoidCoinShort);
	rmAddObjectDefConstraint(MuskOXID, avoidCaribou);
	rmAddObjectDefConstraint(MuskOXID, avoidMoose);
	rmAddObjectDefConstraint(MuskOXID, avoidMuskOx);
	rmAddObjectDefConstraint(MuskOXID, avoidSocket);
	rmAddObjectDefConstraint(MuskOXID, avoidTCFar);
	//rmAddObjectDefConstraint(MuskOXID, PromontoirHunts);
	rmAddObjectDefConstraint(MuskOXID, avoidCliffs);
	rmAddObjectDefConstraint(MuskOXID, stayCenterIsland);
	rmSetObjectDefCreateHerd(MuskOXID, true);
	rmPlaceObjectDefAtLoc(MuskOXID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*2);

 rmSetStatusText("",0.90);
// ------------------------------------ Nuggets ------------------------------------
	int nuggetID1= rmCreateObjectDef("nugget1");
	rmAddObjectDefItem(nuggetID1, "Nugget", 1, 0.0);
	rmSetObjectDefMinDistance(nuggetID1, 0.0);
	rmSetObjectDefMaxDistance(nuggetID1, rmXFractionToMeters(0.5));
	rmAddObjectDefConstraint(nuggetID1, avoidImpassableLandShort);
 	rmAddObjectDefConstraint(nuggetID1, avoidTradeRoute);
	rmAddObjectDefConstraint(nuggetID1, avoidSocket);
    rmAddObjectDefConstraint(nuggetID1, avoidTCFar);
	rmAddObjectDefConstraint(nuggetID1, avoidCliffs);
	rmAddObjectDefConstraint(nuggetID1, stayCenterIsland);
	rmAddObjectDefConstraint(nuggetID1, avoidCoinShort);
  	rmAddObjectDefConstraint(nuggetID1, avoidAll);
  	rmAddObjectDefConstraint(nuggetID1, avoidNugget);
	rmSetNuggetDifficulty(2, 2);
	rmPlaceObjectDefAtLoc(nuggetID1, 0, 0.5, 0.5, cNumberNonGaiaPlayers*2);
		rmSetNuggetDifficulty(1, 1);
	rmPlaceObjectDefAtLoc(nuggetID1, 0, 0.5, 0.5, cNumberNonGaiaPlayers*2);
	
// ------------------------------------------------------ KOTH ---------------------------------------------------------------------

	if(rmGetIsKOTH()) {
    
		int randLoc = rmRandInt(1,3);
		float xLoc = 0.5;
		float yLoc = 0.5;
		float walk = 0.075;

		ypKingsHillPlacer(xLoc, yLoc, walk, 0);
		rmEchoInfo("XLOC = "+xLoc);
		rmEchoInfo("XLOC = "+yLoc);
	}

	 rmSetStatusText("",1.0);


}
