// RM script of Cyprus
// For K&B mod
// By AOE_Fan
// TAD-compatible version!

include "mercenaries.xs";
include "ypAsianInclude.xs";
include "ypKOTHInclude.xs";

void main(void)
{
   // Text
   // Make the loading bar move
   rmSetStatusText("",0.01);


   // Picks the map size
   int playerTiles = 11000;
   if (cNumberNonGaiaPlayers > 4)
      playerTiles = 10000;
   if (cNumberNonGaiaPlayers > 6)
      playerTiles = 8500;

   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);
	
   // Set map smoothness
   rmSetMapElevationHeightBlend(1);


   // Chooses the side where teams start in team games
   int teamSide = rmRandInt(1,2);


   // Picks a default water height
   rmSetSeaLevel(0.0);


   // Lighting
   rmSetLightingSet("sonora");


   // Picks default terrain
   rmSetMapElevationParameters(cElevTurbulence, 0.10, 4, 0.4, 5.0);
   rmSetBaseTerrainMix("araucania_grass_b");
   rmTerrainInitialize("araucania\ground12_ara", 1.0);
   rmSetMapType("grass");
   rmSetMapType("water");
   rmSetMapType("greatPlains");


   chooseMercs();


   // Corner constraint.
   rmSetWorldCircleConstraint(true);

   // Define classes. These are used for constraints
   int classPlayer=rmDefineClass("player");
   int classAnimals=rmDefineClass("animals");
   int classMines=rmDefineClass("mines");
   rmDefineClass("startingUnit");
   rmDefineClass("classForest");
   rmDefineClass("classTreeStraggler");
   rmDefineClass("nuggets");


   // Define constraints
   
   // Map edge constraints
   int circleConstraint=rmCreatePieConstraint("circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.47), rmDegreesToRadians(0), rmDegreesToRadians(360));

   // Player constraints
   int veryLongPlayerConstraint=rmCreateClassDistanceConstraint("nuggets stay away from players very long", classPlayer, 70.0);
   int longPlayerConstraint=rmCreateClassDistanceConstraint("nuggets stay away from players long", classPlayer, 55.0);
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 45.0);
   int mediumPlayerConstraint=rmCreateClassDistanceConstraint("nuggets stay away from players medium", classPlayer, 40.0);
   int somePlayerConstraint=rmCreateClassDistanceConstraint("things stay away from players", classPlayer, 30.0);
   int shortPlayerConstraint=rmCreateClassDistanceConstraint("stay away from players short", classPlayer, 20.0);
   int shortAvoidStartingUnits=rmCreateClassDistanceConstraint("objects avoid starting units short", rmClassID("startingUnit"), 10.0);

   // Resource avoidance
   int forestConstraint=rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 45.0);
   int shortForestConstraint=rmCreateClassDistanceConstraint("forest vs. forest short", rmClassID("classForest"), 10.0);
   int stragglerTreeConstraint=rmCreateClassDistanceConstraint("straggler tree constraint", rmClassID("classTreeStraggler"), 20.0);
   int avoidMines=rmCreateClassDistanceConstraint("avoid mines", classMines, 80.0);
   int avoidStartResource=rmCreateTypeDistanceConstraint("start resource no overlap", "resource", 10.0);
   int animalConstraint=rmCreateClassDistanceConstraint("avoid all animals", classAnimals, 55.0);
   int avoidGoat=rmCreateTypeDistanceConstraint("herds avoids goat", "sheep", 55.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 5.0);

   // Nugget avoidance
   int avoidNuggets=rmCreateClassDistanceConstraint("stuff avoids nuggets", rmClassID("nuggets"), 40.0);

   // Decoration avoidance
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 5.0);

   // Trade route avoidance
   int avoidTradeRoute=rmCreateTradeRouteDistanceConstraint("trade route", 5.0);
   int avoidTradeSockets=rmCreateTypeDistanceConstraint("avoid trade sockets", "SocketTradeRoute", 5.0);

   // Water objects avoidance	
   int fishVsFish=rmCreateTypeDistanceConstraint("fish avoid fish", "ypFishTuna", 25.0);
   int fishLand = rmCreateTerrainDistanceConstraint("fish avoid land", "land", true, 5.0);
   int flagLand = rmCreateTerrainDistanceConstraint("flag avoid land", "land", true, 15.0);
   int flagVsFlag = rmCreateTypeDistanceConstraint("flag avoid flags", "HomeCityWaterSpawnFlag", 25);


   // Text
   rmSetStatusText("",0.10);


   // Define and place water areas

   // Define and place bay  
   int bayID=rmCreateArea("Bay");
   rmSetAreaWaterType(bayID, "new england coast");
   rmSetAreaSize(bayID, 0.1, 0.1);
   rmSetAreaCoherence(bayID, 0.6);
   rmSetAreaLocation(bayID, 0.5, 0.5);
   rmSetAreaBaseHeight(bayID, 0.0);
   rmSetAreaObeyWorldCircleConstraint(bayID, false);
   rmSetAreaMinBlobs(bayID, 3);
   rmSetAreaMaxBlobs(bayID, 3);
   rmSetAreaMinBlobDistance(bayID, 3);
   rmSetAreaMaxBlobDistance(bayID, 3);
   rmSetAreaSmoothDistance(bayID, 20);
   rmBuildArea(bayID);

   // Define and place sea
   int seaID=rmCreateArea("Sea");
   rmSetAreaWaterType(seaID, "new england coast");
   rmSetAreaSize(seaID, 0.2, 0.2);
   rmSetAreaCoherence(seaID, 0.6);
   rmSetAreaLocation(seaID, 0.1, 0.43);
   rmSetAreaBaseHeight(seaID, 0.0);
   rmSetAreaObeyWorldCircleConstraint(seaID, false);
   rmSetAreaMinBlobs(seaID, 20);
   rmSetAreaMaxBlobs(seaID, 20);
   rmSetAreaMinBlobDistance(seaID, 20);
   rmSetAreaMaxBlobDistance(seaID, 20);
   rmSetAreaSmoothDistance(seaID, 20);
   rmBuildArea(seaID);

   // Define and place sea 2
   int sea1ID=rmCreateArea("Sea1");
   rmSetAreaWaterType(sea1ID, "new england coast");
   rmSetAreaSize(sea1ID, 0.2, 0.2);
   rmSetAreaCoherence(sea1ID, 0.6);
   rmSetAreaLocation(sea1ID, 0.2, 0.0);
   rmSetAreaBaseHeight(sea1ID, 0.0);
   rmSetAreaObeyWorldCircleConstraint(sea1ID, false);
   rmSetAreaMinBlobs(sea1ID, 20);
   rmSetAreaMaxBlobs(sea1ID, 20);
   rmSetAreaMinBlobDistance(sea1ID, 20);
   rmSetAreaMaxBlobDistance(sea1ID, 20);
   rmSetAreaSmoothDistance(sea1ID, 20);
   rmBuildArea(sea1ID);

   // Define and place sea 3 - used to take out small bumb
   int sea2ID=rmCreateArea("Sea2");
   rmSetAreaWaterType(sea2ID, "new england coast");
   rmSetAreaSize(sea2ID, 0.01, 0.01);
   rmSetAreaCoherence(sea2ID, 1.0);
   rmSetAreaLocation(sea2ID, 0.5, 0.3);
   rmSetAreaBaseHeight(sea2ID, 0.0);
   rmSetAreaObeyWorldCircleConstraint(sea2ID, false);
   rmSetAreaSmoothDistance(sea2ID, 10);
   rmBuildArea(sea2ID);


   // Text
   rmSetStatusText("",0.20);


   // Place players in FFA and team games

   // Player placement if FFA (Free For All) - place players in circle
   if(cNumberTeams > 2)
   {
      rmSetPlacementSection(0.90, 0.40);
      rmSetTeamSpacingModifier(0.7);
      rmPlacePlayersCircular(0.39, 0.39, 0.0);
   }

   // Player placement if teams - place teams in circle to apart from each other
   if(cNumberTeams == 2)
   {
      rmSetPlacementTeam(0);
      if (rmGetNumberPlayersOnTeam(0) == 1)
      {
         if (teamSide == 1)
            rmSetPlacementSection(0.98, 0.04);
         else
            rmSetPlacementSection(0.30, 0.36);
      }
      else if (rmGetNumberPlayersOnTeam(0) == 2)
      {
         if (teamSide == 1)
            rmSetPlacementSection(0.94, 0.04);
         else
            rmSetPlacementSection(0.24, 0.36);
      }
      else
      {
         if (teamSide == 1)
            rmSetPlacementSection(0.90, 0.08);
         else
            rmSetPlacementSection(0.20, 0.40);
      }
      rmSetTeamSpacingModifier(0.25);
      rmPlacePlayersCircular(0.39, 0.39, 0.0);

      rmSetPlacementTeam(1);
      if (rmGetNumberPlayersOnTeam(1) == 1)
      {
         if (teamSide == 1)
            rmSetPlacementSection(0.30, 0.36);
         else
            rmSetPlacementSection(0.98, 0.04);
      }
      else if (rmGetNumberPlayersOnTeam(1) == 2)
      {
         if (teamSide == 1)
            rmSetPlacementSection(0.24, 0.36);
         else
            rmSetPlacementSection(0.94, 0.04);
      }
      else
      {
         if (teamSide == 1)
            rmSetPlacementSection(0.20, 0.40);
         else
            rmSetPlacementSection(0.90, 0.08);
      }
      rmSetTeamSpacingModifier(0.25);
      rmPlacePlayersCircular(0.39, 0.39, 0.0);
   }


   // Define and place players' areas

   float playerFraction=rmAreaTilesToFraction(100);
   for(i=1; <cNumberPlayers)
   {
      int id=rmCreateArea("Player"+i);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, playerFraction, playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 1);
      rmAddAreaConstraint(id, playerConstraint); 
      rmAddAreaConstraint(id, avoidImpassableLand);
      rmSetAreaLocPlayer(id, i);
      rmSetAreaWarnFailure(id, false);
   }

   // Build the areas.
   rmBuildAllAreas();


   // Text
   rmSetStatusText("",0.30);
		

   // Define starting objects and resources

   int startingTCID=rmCreateObjectDef("startingTC");
   if (rmGetNomadStart())
      rmAddObjectDefItem(startingTCID, "CoveredWagon", 1, 5.0);
   else
      rmAddObjectDefItem(startingTCID, "townCenter", 1, 5.0);
   rmAddObjectDefConstraint(startingTCID, avoidImpassableLand);
   rmAddObjectDefToClass(startingTCID, classPlayer);

   int startingUnits=rmCreateStartingUnitsObjectDef(5.0);
   rmSetObjectDefMinDistance(startingUnits, 5.0);
   rmSetObjectDefMaxDistance(startingUnits, 10.0);
   rmAddObjectDefToClass(startingUnits, rmClassID("startingUnit"));

   int startAreaTreeID=rmCreateObjectDef("starting trees");
   rmAddObjectDefItem(startAreaTreeID, "TreeMadrone", 6, 4.0);
   rmSetObjectDefMinDistance(startAreaTreeID, 14);
   rmSetObjectDefMaxDistance(startAreaTreeID, 18);
   rmAddObjectDefConstraint(startAreaTreeID, avoidStartResource);
   rmAddObjectDefConstraint(startAreaTreeID, avoidImpassableLand);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidStartingUnits);

   int startDeerID=rmCreateObjectDef("starting deer");
   rmAddObjectDefItem(startDeerID, "deer", 6, 6.0);
   rmSetObjectDefCreateHerd(startDeerID, true);
   rmSetObjectDefMinDistance(startDeerID, 14);
   rmSetObjectDefMaxDistance(startDeerID, 18);
   rmAddObjectDefConstraint(startDeerID, avoidStartResource);
   rmAddObjectDefConstraint(startDeerID, avoidImpassableLand);
   rmAddObjectDefConstraint(startDeerID, shortAvoidStartingUnits);

   int startCopperID=rmCreateObjectDef("player copper");
   rmAddObjectDefItem(startCopperID, "MineCopper", 1, 5.0);
   rmSetObjectDefMinDistance(startCopperID, 14);
   rmSetObjectDefMaxDistance(startCopperID, 18);
   rmAddObjectDefConstraint(startCopperID, avoidStartResource);
   rmAddObjectDefConstraint(startCopperID, avoidImpassableLand);
   rmAddObjectDefConstraint(startCopperID, shortAvoidStartingUnits);


   for(i=1; <cNumberPlayers)
   {					
      // Place starting objects and resources
      rmPlaceObjectDefAtLoc(startingTCID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	  if(ypIsAsian(i) && rmGetNomadStart() == false)
         rmPlaceObjectDefAtLoc(ypMonasteryBuilder(i, 1), i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startDeerID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startCopperID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

      // Define and place water flags
      int waterFlagID=rmCreateObjectDef("HC water flag"+i);
      rmAddObjectDefItem(waterFlagID, "HomeCityWaterSpawnFlag", 1, 0.0);
      rmAddClosestPointConstraint(flagVsFlag);
      rmAddClosestPointConstraint(flagLand);		
      vector TCLocation = rmGetUnitPosition(rmGetUnitPlacedOfPlayer(startingTCID, i));
      vector closestPoint = rmFindClosestPointVector(TCLocation, rmXFractionToMeters(1.0));

      rmPlaceObjectDefAtLoc(waterFlagID, i, rmXMetersToFraction(xsVectorGetX(closestPoint)), rmZMetersToFraction(xsVectorGetZ(closestPoint)));
      rmClearClosestPointConstraints();
   }


   // Text
   rmSetStatusText("",0.40);


   // Define and place trade route - runs parallel to the bay

   // Create the trade route
   int tradeRouteID = rmCreateTradeRoute();
   int socketID=rmCreateObjectDef("sockets to dock trade posts");
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID);

   // Define the sockets
   rmAddObjectDefItem(socketID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socketID, true);
   rmSetObjectDefMinDistance(socketID, 0.0);
   rmSetObjectDefMaxDistance(socketID, 12.0);

   // Set the trade route waypoints
   rmAddTradeRouteWaypoint(tradeRouteID, 0.37, 0.72);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.5, 0.75, 1, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.6, 0.74, 1, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.68, 0.68, 1, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.73, 0.6, 1, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.75, 0.5, 1, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.73, 0.4, 1, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.68, 0.3, 1, 1);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.63, 0.28, 1, 1);
   bool placedTradeRoute = rmBuildTradeRoute(tradeRouteID, "dirt");
   if(placedTradeRoute == false)
      rmEchoError("Failed to place trade route"); 
  
   // Place sockets to trade route
   vector socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.1);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.35);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.65);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.9);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);


   // Define and place forests
	
   int numTries=30*cNumberNonGaiaPlayers;
   int failCount=0;
   for (i=0; <numTries)
   {   
      int forest=rmCreateArea("forest"+i);
      rmSetAreaWarnFailure(forest, false);
      rmSetAreaSize(forest, rmAreaTilesToFraction(300), rmAreaTilesToFraction(350));
      rmSetAreaForestType(forest, "california madrone forest");
      rmSetAreaForestDensity(forest, 0.8);
      rmAddAreaToClass(forest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(forest, 0.5);
      rmSetAreaForestUnderbrush(forest, 0.2);
      rmSetAreaBaseHeight(forest, 1.0);
      rmSetAreaCoherence(forest, 0.4);
      rmAddAreaConstraint(forest, avoidImpassableLand);
      rmAddAreaConstraint(forest, somePlayerConstraint);
      rmAddAreaConstraint(forest, forestConstraint);
      rmAddAreaConstraint(forest, avoidTradeRoute);
      rmAddAreaConstraint(forest, avoidTradeSockets);
      if(rmBuildArea(forest)==false)
      {
            // Stop trying once we fail 5 times in a row.  
            failCount++;
            if(failCount==5)
                        break;
      }
      else
                        failCount=0; 
   }


   // Define and place straggler trees
   for (i=0; < 40)
   { 
      int stragglerTreeID=rmCreateObjectDef("straggler tree olive"+i);
      rmAddObjectDefItem(stragglerTreeID, "TreeMadrone", rmRandInt(1,5), 10.0);
      rmSetObjectDefMinDistance(stragglerTreeID, 0.0);
      rmSetObjectDefMaxDistance(stragglerTreeID, rmZFractionToMeters(0.47));
      rmAddObjectDefToClass(stragglerTreeID, rmClassID("classForest"));
      rmAddObjectDefToClass(stragglerTreeID, rmClassID("classTreeStraggler"));
      rmAddObjectDefConstraint(stragglerTreeID, somePlayerConstraint);
      rmAddObjectDefConstraint(stragglerTreeID, avoidImpassableLand);
      rmAddObjectDefConstraint(stragglerTreeID, shortForestConstraint);
      rmAddObjectDefConstraint(stragglerTreeID, stragglerTreeConstraint);
      rmAddObjectDefConstraint(stragglerTreeID, avoidTradeRoute);
      rmAddObjectDefConstraint(stragglerTreeID, avoidTradeSockets);
      rmPlaceObjectDefAtLoc(stragglerTreeID, 0, 0.5, 0.5);
   }


   // Text
   rmSetStatusText("",0.50);


		//============================================================================================\\
		//==================================== BEGIN QUICKBALANCE ====================================\\
		//====================================== qB By Durokan =======================================\\
		//============================================================================================\\
		int quickBalanceCircle=rmCreatePieConstraint("quickBalance circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.48), rmDegreesToRadians(0), rmDegreesToRadians(360));
		int quickBalanceAvoidWaterShort = rmCreateTerrainDistanceConstraint("quickBalance avoid water short", "Land", false, 6.0);
		int quickBalanceAvoidTradeRouteSmall = rmCreateTradeRouteDistanceConstraint("quickBalance objects avoid trade route small", 4.0);
		int quickBalanceAvoidTownCenter=rmCreateTypeDistanceConstraint("quickBalance avoid Town Center", "townCenter", 35.0);
		rmDefineClass("classPlateau");
		int quickBalanceavoidPlateau=rmCreateClassDistanceConstraint("stuff vs. cliffs", rmClassID("classPlateau"), 5.0);
		//rmAddAreaToClass(cliffs, rmClassID("classPlateau"));

		//=========== HUNTS ===========

		int huntRows = 0;
		float huntX = 0.0; 
		float huntXmod = 0.1;
	
		for (huntRows = 0; < (5)) {  
		int qBmainHunts=rmCreateObjectDef("qBhunts"+huntRows);
		rmAddObjectDefItem(qBmainHunts, "deer", rmRandInt(8,8), 10.0);
		rmSetObjectDefCreateHerd(qBmainHunts, true);
		rmSetObjectDefMinDistance(qBmainHunts, 0);
		rmSetObjectDefMaxDistance(qBmainHunts, 16);
		rmAddObjectDefConstraint(qBmainHunts, quickBalanceAvoidWaterShort);
		rmAddObjectDefConstraint(qBmainHunts, quickBalanceAvoidTownCenter);
		rmAddObjectDefConstraint(qBmainHunts, quickBalanceCircle);
		rmAddObjectDefConstraint(qBmainHunts, quickBalanceavoidPlateau);
		if(cNumberNonGaiaPlayers >5){
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .15, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .29, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .43, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .57, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .71, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .85, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .99, 1);
		}else{
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .1, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .3, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .5, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .7, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .9, 1);
		}
		huntX = huntX + .25;
		huntXmod = huntXmod + .25;
		}
		
		//=========== GOLD ===========

		int goldRows = 0;
		float goldZ = 0.0;
		float goldZmod = 0.1;
	
		for (goldRows = 0; < (5)) {  
		int qBmines=rmCreateObjectDef("qBmines"+goldRows);
		rmSetObjectDefMinDistance(qBmines, 0);
		rmSetObjectDefMaxDistance(qBmines, 18);
		rmAddObjectDefConstraint(qBmines, quickBalanceAvoidTownCenter);
		rmAddObjectDefConstraint(qBmines, quickBalanceAvoidWaterShort);
		rmAddObjectDefConstraint(qBmines, quickBalanceCircle);
		rmAddObjectDefConstraint(qBmines, quickBalanceavoidPlateau);
		rmAddObjectDefItem(qBmines, "mine", 1, 0);

		if(cNumberNonGaiaPlayers > 5){
		rmPlaceObjectDefAtLoc(qBmines, 0, .15, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .29, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .43, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .57, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .71, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .85, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .99, goldZ, 1);
		}else{
		rmPlaceObjectDefAtLoc(qBmines, 0, .1, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .3, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .5, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .7, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .9, goldZ, 1);
		}
		goldZ = goldZ + .25;
		goldZmod = goldZmod + .25;
		}

		int nuggetRows = 0;
		float nuggetX = 0.5;
		float nuggetZ = 0.0;
		float nuggetZmod = 0.1;
		float nuggetXmod = 0.1;

		//=========== TREASURES ===========
		for (nuggetRows = 0; < (5)) {  
		int qBtreasures=rmCreateObjectDef("qBtreasures"+nuggetRows);
        rmAddObjectDefItem(qBtreasures, "Nugget", 1, 0.0);
		rmSetObjectDefMinDistance(qBtreasures, 0);
		rmSetObjectDefMaxDistance(qBtreasures, 20);
		rmSetNuggetDifficulty(1, 2);
		rmAddObjectDefConstraint(qBtreasures, quickBalanceAvoidWaterShort);
		rmAddObjectDefConstraint(qBtreasures, quickBalanceCircle);
		rmAddObjectDefConstraint(qBtreasures, quickBalanceavoidPlateau);

		if(cNumberNonGaiaPlayers >5){
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .15, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .29, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .43, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .57, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .71, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .85, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .99, nuggetZmod, 1);
		}else{
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .1, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .3, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .5, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .7, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .9, nuggetZmod, 1);
		}
		
		nuggetZ = nuggetZ + .2;
		nuggetX = nuggetX + .2;
		nuggetZmod = nuggetZmod + .2;
		nuggetXmod = nuggetXmod + .2;
		}		
		//============================================================================================\\
		//===================================== END QUICKBALANCE =====================================\\
		//============================================================================================

   // Text
   rmSetStatusText("",0.70);
	

   // Define and place fish

   int fishID=rmCreateObjectDef("fish");
   rmAddObjectDefItem(fishID, "ypFishTuna", 1, 4.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.47));
   rmAddObjectDefConstraint(fishID, fishVsFish);
   rmAddObjectDefConstraint(fishID, fishLand);
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 10*cNumberNonGaiaPlayers);


   // Text
   rmSetStatusText("",0.80);


   // Define and place treasures

   // Text
   rmSetStatusText("",0.90);


   // Define and place goat herds 

   int goatID=rmCreateObjectDef("goat");
   rmAddObjectDefItem(goatID, "sheep", 2, 4.0);
   rmSetObjectDefMinDistance(goatID, 0.0);
   rmSetObjectDefMaxDistance(goatID, rmXFractionToMeters(0.47));
   rmAddObjectDefConstraint(goatID, avoidGoat);
   rmAddObjectDefConstraint(goatID, playerConstraint);
   rmAddObjectDefConstraint(goatID, avoidImpassableLand);
   rmAddObjectDefConstraint(goatID, avoidTradeRoute);
   rmAddObjectDefConstraint(goatID, avoidTradeSockets);
   rmPlaceObjectDefAtLoc(goatID, 0, 0.5, 0.5, cNumberNonGaiaPlayers);
   

   // Text
   rmSetStatusText("",1.0); 
}