// RM script of Sweden
// For K&B mod
// By AOE_Fan
// TAD-compatible version!

include "mercenaries.xs";
include "ypAsianInclude.xs";
include "ypKOTHInclude.xs";

void main(void)
{
   // Text
   // Make the loading bar move
   rmSetStatusText("",0.01);


   // Picks the map size
   int playerTiles = 11000;
   if (cNumberNonGaiaPlayers > 4)
      playerTiles = 10000;
   if (cNumberNonGaiaPlayers > 6)
      playerTiles = 8500;

   int size=2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmSetMapSize(size, size);
	
   // Set map smoothness
   rmSetMapElevationHeightBlend(1);


   // Chooses the side where teams start in team games
   int teamSide = rmRandInt(1,2);


   // Picks a default water height
   rmSetSeaLevel(0.0);


   // Lighting
   rmSetLightingSet("Great Lakes");


   // Picks default terrain
   rmSetMapElevationParameters(cElevTurbulence, 0.10, 4, 0.4, 7.0);
   rmSetBaseTerrainMix("new_england_forest");
   rmTerrainInitialize("new_england\ground3_ne", 1.0);
   rmSetMapType("grass");
   rmSetMapType("water");
   rmSetMapType("greatPlains");
   

   chooseMercs();


   // Corner constraint.
   rmSetWorldCircleConstraint(true);

   // Define classes. These are used for constraints
   int classPlayer=rmDefineClass("player");
   int classIsland=rmDefineClass("island");
   int classAnimals=rmDefineClass("animals");
   int classMines=rmDefineClass("mines");
   rmDefineClass("startingUnit");
   rmDefineClass("classForest");
   rmDefineClass("nuggets");


   // Define constraints
   
   // Map edge constraints
   int circleConstraint=rmCreatePieConstraint("circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.47), rmDegreesToRadians(0), rmDegreesToRadians(360));

   // Player constraints
   int veryLongPlayerConstraint=rmCreateClassDistanceConstraint("nuggets stay away from players very long", classPlayer, 70.0);
   int longPlayerConstraint=rmCreateClassDistanceConstraint("nuggets stay away from players long", classPlayer, 55.0);
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 45.0);
   int mediumPlayerConstraint=rmCreateClassDistanceConstraint("nuggets stay away from players medium", classPlayer, 40.0);
   int somePlayerConstraint=rmCreateClassDistanceConstraint("things stay away from players", classPlayer, 30.0);
   int shortPlayerConstraint=rmCreateClassDistanceConstraint("stay away from players short", classPlayer, 20.0);
   int shortAvoidStartingUnits=rmCreateClassDistanceConstraint("objects avoid starting units short", rmClassID("startingUnit"), 10.0);

   // Resource avoidance
   int forestConstraint=rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 20.0);
   int avoidMines=rmCreateClassDistanceConstraint("avoid mines", classMines, 60.0);
   int avoidStartResource=rmCreateTypeDistanceConstraint("start resource no overlap", "resource", 10.0);
   int animalConstraint=rmCreateClassDistanceConstraint("avoid all animals", classAnimals, 55.0);
   int avoidCow=rmCreateTypeDistanceConstraint("cows avoids cows", "cow", 55.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 6.0);

   // Nugget avoidance
   int avoidNuggets=rmCreateClassDistanceConstraint("stuff avoids nuggets", rmClassID("nuggets"), 60.0);

   // Area avoidance
   int islandConstraint=rmCreateClassDistanceConstraint("avoid island", classIsland, 20.0);

   // Decoration avoidance
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 5.0);

   // Trade route avoidance
   int avoidTradeRoute=rmCreateTradeRouteDistanceConstraint("trade route", 5.0);
   int avoidTradeSockets=rmCreateTypeDistanceConstraint("avoid trade sockets", "SocketTradeRoute", 5.0);

   // Water objects avoidance	
   int fishVsFish=rmCreateTypeDistanceConstraint("fish v fish", "FishSalmon", 20.0);
   int fishLand = rmCreateTerrainDistanceConstraint("fish land", "land", true, 5.0);
   int flagLand = rmCreateTerrainDistanceConstraint("flag vs land", "land", true, 15.0);
   int flagVsFlag = rmCreateTypeDistanceConstraint("flag avoid same", "HomeCityWaterSpawnFlag", 25);


   // Text
   rmSetStatusText("",0.10);


   // Define and place Vnern to middle

   int lakeID=rmCreateArea("Lake");
   rmSetAreaWaterType(lakeID, "great lakes");
   rmSetAreaSize(lakeID, 0.12, 0.12);
   rmSetAreaCoherence(lakeID, 0.8);
   rmSetAreaLocation(lakeID, 0.5, 0.5);
   rmSetAreaBaseHeight(lakeID, 0.0);
   rmSetAreaMinBlobs(lakeID, 10);
   rmSetAreaMaxBlobs(lakeID, 20);
   rmSetAreaMinBlobDistance(lakeID, 10);
   rmSetAreaMaxBlobDistance(lakeID, 20);
   rmSetAreaSmoothDistance(lakeID, 10);
   rmBuildArea(lakeID);


   // Define and place island to lake IF it's KOTH or FFA game!
   if (rmGetIsFFA() || rmGetIsKOTH()) 
   {
      int islandID=rmCreateArea("Island");
      rmSetAreaTerrainType(islandID, "nwterritory\ground_grass3_nwt");
      rmSetAreaMix(islandID, "nwt_grass1");
      if (cNumberNonGaiaPlayers <= 3)
            rmSetAreaSize(islandID, 0.007, 0.007);
      if (cNumberNonGaiaPlayers >= 4)
            rmSetAreaSize(islandID, 0.006, 0.006);
      if (cNumberNonGaiaPlayers >= 6)
            rmSetAreaSize(islandID, 0.005, 0.005);
      rmSetAreaCoherence(islandID, 0.6);
      rmSetAreaLocation(islandID, 0.5, 0.5);
      rmSetAreaBaseHeight(islandID, 1.0);
      rmSetAreaMinBlobs(islandID, 4);
      rmSetAreaMaxBlobs(islandID, 5);
      rmSetAreaMinBlobDistance(islandID, 4);
      rmSetAreaMaxBlobDistance(islandID, 5);
      rmSetAreaSmoothDistance(islandID, 5);
      rmAddAreaToClass(islandID, classIsland);
      rmBuildArea(islandID);
   }


   // Text
   rmSetStatusText("",0.40);


   // Place players - in team and FFA games

   // Player placement if FFA (Free For All) - place players in circle
   if(cNumberTeams > 2)
   {
      rmSetTeamSpacingModifier(0.7);
      rmPlacePlayersCircular(0.36, 0.36, 0.0);
   }

   // Player placement if teams - place teams in circle to apart from each other
   if(cNumberTeams == 2)
   {
      rmSetPlacementTeam(0);
      if (teamSide == 1)
            rmSetPlacementSection(0.00, 0.25);
      else if (teamSide == 2)
            rmSetPlacementSection(0.50, 0.75);
      rmSetTeamSpacingModifier(0.25);
      rmPlacePlayersCircular(0.36, 0.36, 0.0);

      rmSetPlacementTeam(1);
      if (teamSide == 1)
            rmSetPlacementSection(0.50, 0.75);
      else if (teamSide == 2)
            rmSetPlacementSection(0.00, 0.25);
      rmSetTeamSpacingModifier(0.25);
      rmPlacePlayersCircular(0.36, 0.36, 0.0);
   }


   // Define and place players' areas

   float playerFraction=rmAreaTilesToFraction(100);
   for(i=1; <cNumberPlayers)
   {
      int id=rmCreateArea("Player"+i);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, playerFraction, playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 1);
      rmAddAreaConstraint(id, playerConstraint); 
      rmAddAreaConstraint(id, avoidImpassableLand);
      rmSetAreaLocPlayer(id, i);
      rmSetAreaWarnFailure(id, false);
   }

   // Build the areas.
   rmBuildAllAreas();


   // Text
   rmSetStatusText("",0.50);
		

   // Define starting objects and resources

   int startingTCID=rmCreateObjectDef("startingTC");
   if (rmGetNomadStart())
      rmAddObjectDefItem(startingTCID, "CoveredWagon", 1, 5.0);
   else
      rmAddObjectDefItem(startingTCID, "townCenter", 1, 5.0);
   rmAddObjectDefConstraint(startingTCID, avoidImpassableLand);
   rmAddObjectDefToClass(startingTCID, classPlayer);

   int startingUnits=rmCreateStartingUnitsObjectDef(5.0);
   rmSetObjectDefMinDistance(startingUnits, 5.0);
   rmSetObjectDefMaxDistance(startingUnits, 10.0);
   rmAddObjectDefToClass(startingUnits, rmClassID("startingUnit"));

   int startAreaTreeID=rmCreateObjectDef("starting trees");
   rmAddObjectDefItem(startAreaTreeID, "TreeGreatLakes", 6, 4.0);
   rmSetObjectDefMinDistance(startAreaTreeID, 12);
   rmSetObjectDefMaxDistance(startAreaTreeID, 16);
   rmAddObjectDefConstraint(startAreaTreeID, avoidStartResource);
   rmAddObjectDefConstraint(startAreaTreeID, avoidImpassableLand);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidStartingUnits);

   int startMooseID=rmCreateObjectDef("starting moose");
   rmAddObjectDefItem(startMooseID, "moose", 5, 4.0);
   rmSetObjectDefCreateHerd(startMooseID, true);
   rmSetObjectDefMinDistance(startMooseID, 14);
   rmSetObjectDefMaxDistance(startMooseID, 18);
   rmAddObjectDefConstraint(startMooseID, avoidStartResource);
   rmAddObjectDefConstraint(startMooseID, avoidImpassableLand);
   rmAddObjectDefConstraint(startMooseID, shortAvoidStartingUnits);

   int startSilverID=rmCreateObjectDef("player silver");
   rmAddObjectDefItem(startSilverID, "Mine", 1, 5.0);
   rmSetObjectDefMinDistance(startSilverID, 14);
   rmSetObjectDefMaxDistance(startSilverID, 18);
   rmAddObjectDefConstraint(startSilverID, avoidStartResource);
   rmAddObjectDefConstraint(startSilverID, avoidImpassableLand);
   rmAddObjectDefConstraint(startSilverID, shortAvoidStartingUnits);


   for(i=1; <cNumberPlayers)
   {					
      // Place starting objects and resources
      rmPlaceObjectDefAtLoc(startingTCID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
	  if(ypIsAsian(i) && rmGetNomadStart() == false)
         rmPlaceObjectDefAtLoc(ypMonasteryBuilder(i, 1), i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startMooseID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startSilverID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));

      // Define and place water flags
      int waterFlagID=rmCreateObjectDef("HC water flag"+i);
      rmAddObjectDefItem(waterFlagID, "HomeCityWaterSpawnFlag", 1, 0.0);
      rmAddClosestPointConstraint(flagVsFlag);
      rmAddClosestPointConstraint(flagLand);		
      vector TCLocation = rmGetUnitPosition(rmGetUnitPlacedOfPlayer(startingTCID, i));
      vector closestPoint = rmFindClosestPointVector(TCLocation, rmXFractionToMeters(1.0));

      rmPlaceObjectDefAtLoc(waterFlagID, i, rmXMetersToFraction(xsVectorGetX(closestPoint)), rmZMetersToFraction(xsVectorGetZ(closestPoint)));
      rmClearClosestPointConstraints();
   }


   // Text
   rmSetStatusText("",0.70);


   // Define and place trade routes

   // Create the trade routes
   int tradeRouteID = rmCreateTradeRoute();
   int socketID=rmCreateObjectDef("sockets to dock trade posts");
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID);

   int tradeRoute2ID = rmCreateTradeRoute();
   int socket2ID=rmCreateObjectDef("sockets to dock trade posts 2");
   rmSetObjectDefTradeRouteID(socket2ID, tradeRoute2ID);

   // Define the sockets
   rmAddObjectDefItem(socketID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socketID, true);
   rmSetObjectDefMinDistance(socketID, 0.0);
   rmSetObjectDefMaxDistance(socketID, 12.0);

   rmAddObjectDefItem(socket2ID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socket2ID, true);
   rmSetObjectDefMinDistance(socket2ID, 0.0);
   rmSetObjectDefMaxDistance(socket2ID, 12.0);

   // Set the trade route waypoints
   rmAddTradeRouteWaypoint(tradeRouteID, 1.0, 0.3);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.7, 0.25, 8, 8);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.65, 0.0, 8, 8);
   bool placedTradeRoute = rmBuildTradeRoute(tradeRouteID, "dirt");
   if(placedTradeRoute == false)
      rmEchoError("Failed to place trade route");

   rmAddTradeRouteWaypoint(tradeRoute2ID, 0.3, 1.0);
   rmAddRandomTradeRouteWaypoints(tradeRoute2ID, 0.25, 0.7, 8, 8);
   rmAddRandomTradeRouteWaypoints(tradeRoute2ID, 0.0, 0.65, 8, 8);
   placedTradeRoute = rmBuildTradeRoute(tradeRoute2ID, "dirt");
   if(placedTradeRoute == false)
      rmEchoError("Failed to place trade route 2"); 
  
   // Place sockets to trade routes
   vector socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.1);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);
   socketLoc = rmGetTradeRouteWayPoint(tradeRouteID, 0.9);
   rmPlaceObjectDefAtPoint(socketID, 0, socketLoc);

   socketLoc = rmGetTradeRouteWayPoint(tradeRoute2ID, 0.1);
   rmPlaceObjectDefAtPoint(socket2ID, 0, socketLoc);
   socketLoc = rmGetTradeRouteWayPoint(tradeRoute2ID, 0.9);
   rmPlaceObjectDefAtPoint(socket2ID, 0, socketLoc);


   // Define and place forests
	
   int numTries=500*cNumberNonGaiaPlayers;
   int failCount=0;
   for (i=0; <numTries)
   {   
      int forest=rmCreateArea("forest"+i);
      rmSetAreaWarnFailure(forest, false);
      rmSetAreaSize(forest, rmAreaTilesToFraction(650), rmAreaTilesToFraction(700));
      rmSetAreaForestType(forest, "great lakes forest");
      rmSetAreaForestDensity(forest, 1.0);
      rmAddAreaToClass(forest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(forest, 1.0);
      rmSetAreaForestUnderbrush(forest, 0.3);
      rmSetAreaBaseHeight(forest, 1.0);
      rmSetAreaMinBlobs(forest, 1);
      rmSetAreaMaxBlobs(forest, 3);
      rmSetAreaMinBlobDistance(forest, 10);
      rmSetAreaMaxBlobDistance(forest, 15);
      rmSetAreaCoherence(forest, 0.4);
      rmAddAreaConstraint(forest, avoidImpassableLand);
      rmAddAreaConstraint(forest, somePlayerConstraint);
      rmAddAreaConstraint(forest, forestConstraint);
      rmAddAreaConstraint(forest, islandConstraint);
      rmAddAreaConstraint(forest, avoidTradeRoute);
      rmAddAreaConstraint(forest, avoidTradeSockets);
      if(rmBuildArea(forest)==false)
      {
            // Stop trying once we fail 10 times in a row.  
            failCount++;
            if(failCount==10)
                        break;
      }
      else
                        failCount=0; 
   }


   // Text
   rmSetStatusText("",0.80);


		//============================================================================================\\
		//==================================== BEGIN QUICKBALANCE ====================================\\
		//====================================== qB By Durokan =======================================\\
		//============================================================================================\\
		int quickBalanceCircle=rmCreatePieConstraint("quickBalance circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.48), rmDegreesToRadians(0), rmDegreesToRadians(360));
		int quickBalanceAvoidWaterShort = rmCreateTerrainDistanceConstraint("quickBalance avoid water short", "Land", false, 6.0);
		int quickBalanceAvoidTradeRouteSmall = rmCreateTradeRouteDistanceConstraint("quickBalance objects avoid trade route small", 4.0);
		int quickBalanceAvoidTownCenter=rmCreateTypeDistanceConstraint("quickBalance avoid Town Center", "townCenter", 35.0);
		rmDefineClass("classPlateau");
		int quickBalanceavoidPlateau=rmCreateClassDistanceConstraint("stuff vs. cliffs", rmClassID("classPlateau"), 5.0);
		//rmAddAreaToClass(cliffs, rmClassID("classPlateau"));

		//=========== HUNTS ===========

		int huntRows = 0;
		float huntX = 0.0; 
		float huntXmod = 0.1;
	
		for (huntRows = 0; < (5)) {  
		int qBmainHunts=rmCreateObjectDef("qBhunts"+huntRows);
		rmAddObjectDefItem(qBmainHunts, "elk", rmRandInt(8,8), 10.0);
		rmSetObjectDefCreateHerd(qBmainHunts, true);
		rmSetObjectDefMinDistance(qBmainHunts, 0);
		rmSetObjectDefMaxDistance(qBmainHunts, 16);
		rmAddObjectDefConstraint(qBmainHunts, quickBalanceAvoidWaterShort);
		rmAddObjectDefConstraint(qBmainHunts, quickBalanceAvoidTownCenter);
		rmAddObjectDefConstraint(qBmainHunts, quickBalanceCircle);
		rmAddObjectDefConstraint(qBmainHunts, quickBalanceavoidPlateau);
		if(cNumberNonGaiaPlayers >5){
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .15, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .29, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .43, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .57, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .71, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .85, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .99, 1);
		}else{
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .1, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .3, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .5, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntXmod, .7, 1);
		rmPlaceObjectDefAtLoc(qBmainHunts, 0, huntX, .9, 1);
		}
		huntX = huntX + .25;
		huntXmod = huntXmod + .25;
		}
		
		//=========== GOLD ===========

		int goldRows = 0;
		float goldZ = 0.0;
		float goldZmod = 0.1;
	
		for (goldRows = 0; < (5)) {  
		int qBmines=rmCreateObjectDef("qBmines"+goldRows);
		rmSetObjectDefMinDistance(qBmines, 0);
		rmSetObjectDefMaxDistance(qBmines, 18);
		rmAddObjectDefConstraint(qBmines, quickBalanceAvoidTownCenter);
		rmAddObjectDefConstraint(qBmines, quickBalanceAvoidWaterShort);
		rmAddObjectDefConstraint(qBmines, quickBalanceCircle);
		rmAddObjectDefConstraint(qBmines, quickBalanceavoidPlateau);
		rmAddObjectDefItem(qBmines, "mine", 1, 0);

		if(cNumberNonGaiaPlayers > 5){
		rmPlaceObjectDefAtLoc(qBmines, 0, .15, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .29, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .43, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .57, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .71, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .85, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .99, goldZ, 1);
		}else{
		rmPlaceObjectDefAtLoc(qBmines, 0, .1, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .3, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .5, goldZ, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .7, goldZmod, 1);
		rmPlaceObjectDefAtLoc(qBmines, 0, .9, goldZ, 1);
		}
		goldZ = goldZ + .25;
		goldZmod = goldZmod + .25;
		}

		int nuggetRows = 0;
		float nuggetX = 0.5;
		float nuggetZ = 0.0;
		float nuggetZmod = 0.1;
		float nuggetXmod = 0.1;

		//=========== TREASURES ===========
		for (nuggetRows = 0; < (5)) {  
		int qBtreasures=rmCreateObjectDef("qBtreasures"+nuggetRows);
        rmAddObjectDefItem(qBtreasures, "Nugget", 1, 0.0);
		rmSetObjectDefMinDistance(qBtreasures, 0);
		rmSetObjectDefMaxDistance(qBtreasures, 20);
		rmSetNuggetDifficulty(1, 2);
		rmAddObjectDefConstraint(qBtreasures, quickBalanceAvoidWaterShort);
		rmAddObjectDefConstraint(qBtreasures, quickBalanceCircle);
		rmAddObjectDefConstraint(qBtreasures, quickBalanceavoidPlateau);

		if(cNumberNonGaiaPlayers >5){
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .15, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .29, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .43, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .57, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .71, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .85, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .99, nuggetZmod, 1);
		}else{
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .1, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .3, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .5, nuggetZmod, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .7, nuggetZ, 1);
		rmPlaceObjectDefAtLoc(qBtreasures, 0, .9, nuggetZmod, 1);
		}
		
		nuggetZ = nuggetZ + .2;
		nuggetX = nuggetX + .2;
		nuggetZmod = nuggetZmod + .2;
		nuggetXmod = nuggetXmod + .2;
		}		
		//============================================================================================\\
		//===================================== END QUICKBALANCE =====================================\\
		//============================================================================================   


   // Text
   rmSetStatusText("",0.90);
	

   // Define and place fish

   int fishID=rmCreateObjectDef("fish");
   rmAddObjectDefItem(fishID, "FishSalmon", 1, 4.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(fishID, fishVsFish);
   rmAddObjectDefConstraint(fishID, fishLand);
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 5*cNumberNonGaiaPlayers);


   // Text
   rmSetStatusText("",0.99);


   // Place King's Hill if KOTH game to middle - stays on island we created before

   if(rmGetIsKOTH()) 
   {
      float xLoc = 0.5;
      float yLoc = 0.5;
      float walk = 0.01;

      ypKingsHillPlacer(xLoc, yLoc, walk, 0);
   }


   // Text
   rmSetStatusText("",1.0); 
}  
